import { createPriorityQueue } from '@algorithm.ts/priority-queue';

const ZERO = 0n;
const DEFAULT_INF = BigInt(Math.floor(Number.MAX_SAFE_INTEGER / 2));
const Q = createPriorityQueue((x, y) => {
    if (x.cost === y.cost)
        return 0;
    return x.cost < y.cost ? 1 : -1;
});
function dijkstra(graph, options = {}) {
    const { N, source, edges, G, dist = [] } = graph;
    const { INF = DEFAULT_INF } = options;
    if (dist.length < N)
        dist.length = N;
    dist.fill(INF, 0, N);
    Q.init();
    Q.enqueue({ pos: source, cost: ZERO });
    dist[source] = ZERO;
    while (Q.size() > 0) {
        const { pos, cost } = Q.dequeue();
        if (dist[pos] < cost)
            continue;
        for (const idx of G[pos]) {
            const edge = edges[idx];
            const candidate = dist[pos] + edge.cost;
            if (dist[edge.to] > candidate) {
                dist[edge.to] = candidate;
                Q.enqueue({ pos: edge.to, cost: dist[edge.to] });
            }
        }
    }
    return dist;
}

export { dijkstra as default, dijkstra };
